@@ -43,6 +43,15 @@ CIPHERS = (
 )
 
 
+TRACE_ID_HEADER = 'X-B3-TraceId'  # type: str
+
+
+def fresh_trace_id():
+    # type: () -> str
+    # returns a string which is a valid zipkin trace id
+    return '{:02x}'.format(random.getrandbits(64))
+
+
 class Service(object):
     _requests_session = None  # type: requests.Session
     _uris = None  # type: List[str]
@@ -81,6 +90,10 @@ class Service(object):
 
         kwargs['timeout'] = (self._connect_timeout, self._read_timeout)
 
+        if 'headers' not in kwargs:
+            kwargs['headers'] = {}
+        kwargs['headers'][TRACE_ID_HEADER] = fresh_trace_id()
+
         _response = self._requests_session.request(*args, **kwargs)
         try:
             _response.raise_for_status()
@@ -98,6 +111,12 @@ class Service(object):
         )
 
 
+class RetryWithJitter(Retry):
+    def get_backoff_time(self):
+        jitter = random.random()
+        return jitter * super(RetryWithJitter, self).get_backoff_time()
+
+
 class RequestsClient(object):
 
     @classmethod
@@ -105,7 +124,7 @@ class RequestsClient(object):
         # type: (Type[T], str, ServiceConfiguration) -> T
         # setup retry to match java remoting
         # https://github.com/palantir/http-remoting/tree/3.12.0#quality-of-service-retry-failover-throttling
-        retry = Retry(
+        retry = RetryWithJitter(
             total=service_config.max_num_retries,
             status_forcelist=[308, 429, 503],
             backoff_factor=float(service_config.backoff_slot_size) / 1000,
